//
//  NSData+AESCrypt.h
//
//  AES128Encryption + Base64Encoding
//  @tharindu http://tharindufit.wordpress.com
//
//

#import "NSData+AESCrypt.h"
#import <CommonCrypto/CommonCryptor.h>

@implementation LPAES
+ (NSData *)AES128Operation:(CCOperation)operation key:(NSString *)key iv:(NSString *)iv obj:(NSData *)obj
{
    // Note: The key will be 0's but we intentionally are keeping it this way to maintain
    // compatibility. The correct code is:
    // char keyPtr[[key length] + 1];
    char keyPtr[kCCKeySizeAES128 + 1];
    memset(keyPtr, 0, sizeof(keyPtr));
    [key getCString:keyPtr maxLength:sizeof(keyPtr) encoding:NSUTF8StringEncoding];
    
    char ivPtr[kCCBlockSizeAES128 + 1];
    memset(ivPtr, 0, sizeof(ivPtr));
    [iv getCString:ivPtr maxLength:sizeof(ivPtr) encoding:NSUTF8StringEncoding];
    
    NSUInteger dataLength = [obj length];
    size_t bufferSize = dataLength + kCCBlockSizeAES128;
    void *buffer = malloc(bufferSize);
    
    size_t numBytesCrypted = 0;
    CCCryptorStatus cryptStatus = CCCrypt(operation,
                                          kCCAlgorithmAES128,
                                          kCCOptionPKCS7Padding,
                                          keyPtr,
                                          kCCBlockSizeAES128,
                                          ivPtr,
                                          [obj bytes],
                                          dataLength,
                                          buffer,
                                          bufferSize,
                                          &numBytesCrypted);
    if (cryptStatus == kCCSuccess) {
        return [NSData dataWithBytesNoCopy:buffer length:numBytesCrypted];
    }
    free(buffer);
    return nil;
}

+ (NSData *)AES128EncryptWithKey:(NSString *)key iv:(NSString *)iv obj:(NSData *)obj
{
    return [LPAES AES128Operation:kCCEncrypt key:key iv:iv obj:obj];
}

+ (NSData *)AES128DecryptWithKey:(NSString *)key iv:(NSString *)iv obj:(NSData *)obj
{
    return [LPAES AES128Operation:kCCDecrypt key:key iv:iv obj:obj];
}
@end
